; Functions for handling null-terminated strings

proc ConcatStr, .pDest, .pSource                                ; concatenates the null-terminated string at pSource to the end of the string at pDest
begin                                                           ; return value is the number of characters added to the destination string
                                                                ; NOTE: make DAMN SURE that there is enough space in pDest for the combined string first!
        push    esi
        push    edi

        mov     esi,[.pSource]
        mov     edi,[.pDest]

        stdcall GetStrLen, ESI

        test    eax,eax
        jz      .exit                                           ; no point in attempting to copy a 0-length string...

        push    eax

        stdcall GetStrLen, EDI

        add     edi,eax

        stdcall CopyStr, EDI, ESI

        pop     eax

.exit:

        pop     edi
        pop     esi

        return

endp


proc ConvertStrToLowercase, .pStr                               ; converts a null-terminated string at pStr to all lowercase characters
begin                                                           ; return value is 0 if the function failed because the string had a length of 0; returns 1 otherwise

        push    esi

        mov     esi,[.pStr]

        stdcall GetStrLen, ESI

        test    eax,eax
        jz      .exit                                           ; can't convert anything inside a 0-length string!

        mov     ecx,eax

.loop:

        mov     al,[esi]

        cmp     al,40h                                          ; we are only interested in uppercase ASCII characters here
        jbe     .loop2

        cmp     al,5Ah
        ja      .loop2

        add     al,20h                                          ; converts an uppercase character to a lowercase one
        mov     [esi],al

.loop2:

        inc     esi

        loop    .loop

        mov     eax,1

.exit:

        pop     esi

        return

endp

proc CopyStr, .pDest, .pSource                                  ; copies a null-terminated string from pSource to pDest; returns 1 if successful, 0 otherwise
begin                                                           ; NOTE: make DAMN SURE that there is enough space in pDest for the source string first!

        push    esi
        push    edi

        mov     esi,[.pSource]
        mov     edi,[.pDest]

        stdcall GetStrLen, ESI

        test    eax,eax
        jz      .exit                                           ; no point in attempting to copy a 0-length string...

        inc     eax                                             ; also copy the null terminator!

        mov     ecx,eax
        and     eax,3
        shr     ecx,2
        jecxz   .copy2

        rep     movsd

.copy2:

        mov     ecx,eax

        rep     movsb

.exit:

        pop     edi
        pop     esi

        return

endp


proc FindCharInStr, .pStr, .Char                                ; the returned value indicates success/failure - it will be zero if no match was found!
begin                                                           ; if returned value of EAX is nonzero, then ECX contains the offset to the first found instance of the specified character
                                                                ; and EDX contains the pointer to the first found instance of the specified character
        push    edi                                             ; NOTE: this function is CASE-SENSITIVE!

        mov     edi,[.pStr]

        stdcall GetStrLen, EDI                                  ; get the string's length first, so we don't end up searching past the end of the string!

        test    eax,eax
        jz      .exit                                           ; there's no point in searching for anything inside a 0-length string!

        mov     ecx,eax
        mov     edx,eax
        mov     eax,[.Char]

        inc     ecx                                             ; required so that we can tell if the search failed...

        repne   scasb                                           ; search for the specified character

        xor     eax,eax
        jecxz   .exit                                           ; will be 0 if no match was found 

        inc     eax
        sub     edx,ecx
        mov     ecx,edx
        add     edx,[.pStr]

.exit:

        pop     edi

        return

endp


proc FindSubstrInStr, .pStr, .pSubStr                           ; the returned value indicates success/failure - it will be zero if no match was found!
                                                                ; if returned value of EAX is nonzero, then ECX contains the offset to the first found instance of string SubStr found in the specified string
        .StrLen         dd ?                                    ; and EDX contains the pointer to the first found instance of string SubStr found in the specified string
        .SubStrLen      dd ?                                    ; NOTE: this function is CASE-SENSITIVE!

begin

        push    esi
        push    edi

        mov     edi,[.pStr]

        stdcall GetStrLen, EDI

        test    eax,eax
        jz      .exit                                           ; there's no point in searching for anything inside a 0-length string!

        mov     [.StrLen],eax

        mov     esi,[.pSubStr]

        stdcall GetStrLen, ESI

        test    eax,eax
        jz      .exit                                           ; there's no point in searching for a 0-length string!

        mov     [.SubStrLen],eax

        mov     ecx,eax
        xor     eax,eax
        cmp     [.StrLen],ecx
        jb      .exit                                           ; no point in searching for a longer substring inside a shorter string!

        dec     edi

.search:

        inc     edi

        mov     eax,[.StrLen]
        add     eax,[.pStr]                                     ; EAX now points to the null terminator of Str
        sub     eax,edi                                         ; EAX = number of characters of Str remaining for the compare operation

        cmp     eax,[.SubStrLen]
        jae     .search2

        xor     eax,eax
        jmp     .exit                                           ; no point in searching any further, when there aren't enough characters remaining to possibly find a match!

.search2:

        stdcall FindCharInStr, EDI, [ESI]                       ; search for the first character of SubStr inside Str

        test    eax,eax
        jz      .exit

        mov     ecx,[.SubStrLen]
        mov     edi,edx

        push    esi
        push    edi

        repe    cmpsb

        pop     edi
        pop     esi

        test    ecx,ecx
        jnz     .search                                         ; if no full match of SubStr was found at this position, advance and try again

        mov     ecx,edx
        sub     ecx,[.pStr]

.exit:

        pop     edi
        pop     esi

        return

endp


proc GetStrLen, .pStr                                           ; returns the length of a zero-terminated string, EXCLUDING the null terminator
begin

        push    edi

        xor     eax,eax
        or      ecx,0FFFFFFFFh
        mov     edi,[.pStr]

        repne   scasb                                           ; search for the null terminator

        not     ecx
        dec     ecx                                             ; we don't want to include the null terminator in the returned length!
        mov     eax,ecx

        pop     edi

        return

endp


proc ReplaceCharsInStr, .pStr, .pCharsToReplace, .NewChar       ; replaces, in the string at pStr, all instances of ANY of the characters contained in the string at pCharsToReplace, with NewChar
                                                                ; returns 0 if the function failed because one or more of the strings had a length of 0; otherwise returns 1
        .StrLen             dd ?                                ; NOTE: this function is CASE-SENSITIVE!
        ;.LoopCtr            dd ?
        ;.CharsToReplaceLen  dd ?

begin

        push    ebx
        push    esi
        push    edi

        mov     esi,[.pCharsToReplace]
        mov     edi,[.pStr]

        stdcall GetStrLen, ESI

        test    eax,eax
        jz      .exit                                           ; can't compare characters against an empty set!

        mov     ebx,eax

        stdcall GetStrLen, EDI

        test    eax,eax
        jz      .exit                                           ; can't compare characters inside a 0-length string!

        mov     [.StrLen],eax

.loop_outer:

        dec     ebx                                             ; iterate over each of the characters at pCharsToReplace

        push    esi

        lea     esi,[esi + ebx]
        mov     edi,[.pStr]

        xor     ecx,ecx
        ;xor     eax,eax
        ;mov     [.LoopCtr],eax

.loop_inner:

        lea     eax,[edi + ecx]

        stdcall FindCharInStr, EAX, [ESI]

        test    eax,eax
        jz      .loop_outer_end                                 ; if no further matches were found

        mov     eax,[.NewChar]
        mov     [edx],al                                        ; replace the character if a match was found

        inc     ecx

.loop_inner_end:

        cmp     ecx,[.StrLen]                                   ; iterate over all of the characters in the string at pStr
        jb     .loop_inner

.loop_outer_end:

        pop     esi

        test    ebx,ebx
        jnz     .loop_outer

        mov     eax,1

.exit:

        pop     edi
        pop     esi
        pop     ebx

        return

endp